@@ -380,6 +380,36 @@ _string_array_to_char_array (char	*dst,
 }
 #endif
 
+#if defined (HAVE_COGL_1_8) && !defined (HAVE_CLUTTER_OSX)
+static gint
+get_n_fragment_texture_units (void)
+{
+  ClutterBackend *backend;
+  CoglContext *context;
+  CoglDisplay *display;
+  CoglRenderer *renderer;
+  gint n;
+
+  backend = clutter_get_default_backend ();
+  context = clutter_backend_get_cogl_context (backend);
+  display = cogl_context_get_display (context);
+  renderer = cogl_display_get_renderer (display);
+
+  n = cogl_renderer_get_n_fragment_texture_units (renderer);
+
+  return n;
+}
+#else
+static gint
+get_n_fragment_texture_units (void)
+{
+  gint n_texture_units;
+
+  glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &n_texture_units);
+  return n_texture_units;
+}
+#endif
+
 static CoglHandle
 _create_cogl_program (const char *source)
 {
@@ -796,7 +826,7 @@ clutter_gst_build_renderers_list (void)
       NULL
     };
 
-  glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &nb_texture_units);
+  nb_texture_units = get_n_fragment_texture_units();
 
   if (nb_texture_units >= 3)
     features |= CLUTTER_GST_MULTI_TEXTURE;
@@ -907,21 +937,38 @@ navigation_event (ClutterActor        *actor,
       gst_navigation_send_mouse_event (GST_NAVIGATION (sink),
                                        type, bevent->button, bevent->x, bevent->y);
     }
-  else if (event->type == CLUTTER_KEY_PRESS ||
-           event->type == CLUTTER_KEY_RELEASE)
+  else if (event->type == CLUTTER_KEY_PRESS)
     {
-      ClutterKeyEvent *kevent = (ClutterKeyEvent *) kevent;
-      const char *type;
-      char *key;
+      ClutterKeyEvent *kevent = (ClutterKeyEvent *) event;
+      GstNavigationCommand command;
 
-      type = (event->type == CLUTTER_KEY_PRESS) ? "key-press" : "key-release";
-      key = g_ucs4_to_utf8 (&kevent->unicode_value, 1, NULL, NULL, NULL);
-      GST_DEBUG ("Received key %s event (%s)",
-                 (event->type == CLUTTER_KEY_PRESS) ? "press" : "release",
-                 key);
-      gst_navigation_send_key_event (GST_NAVIGATION (sink),
-                                     type, key ? key : "unknown");
-      g_free (key);
+      switch (kevent->keyval)
+        {
+        case CLUTTER_KEY_Up:
+          command = GST_NAVIGATION_COMMAND_UP;
+          break;
+        case CLUTTER_KEY_Down:
+          command = GST_NAVIGATION_COMMAND_DOWN;
+          break;
+        case CLUTTER_KEY_Left:
+          command = GST_NAVIGATION_COMMAND_LEFT;
+          break;
+        case CLUTTER_KEY_Right:
+          command = GST_NAVIGATION_COMMAND_RIGHT;
+          break;
+        case CLUTTER_KEY_Return:
+          command = GST_NAVIGATION_COMMAND_ACTIVATE;
+          break;
+        default:
+          command = GST_NAVIGATION_COMMAND_INVALID;
+        }
+
+      if (command != GST_NAVIGATION_COMMAND_INVALID)
+        {
+          gst_navigation_send_command (GST_NAVIGATION (sink), command);
+
+          return TRUE;
+        }
     }
 
   return FALSE;
